/*
 *  Vector.h
 *  Assignment1
 *
 *  Created by Zach Gildersleeve on 1/14/07.
 *
 */
#ifndef VECTOR_H
#define VECTOR_H

#include "Point.h"
class Point;

class Vector {
	public:	
		float x, y, z;
		                  
		Vector();
		Vector(const Vector &a);
		Vector(float newX, float newY, float newZ);
		
		~Vector();
		
		bool operator==(const Vector &a);
		bool operator!=(const Vector &a);
		
		const Vector &operator=(const Vector &a);
		
		float length();
		float lengthSquared();
		
		Vector operator-();
				
		void normalize();
		
		Point asPoint();
};

//Vector operator+(Vector a, Vector b);
inline Vector operator+(const Vector a, const Vector b) {
	return Vector(a.x + b.x, a.y + b.y, a.z + b.z);
}

inline Vector operator-(const Vector a, const Vector b) {
	return Vector(a.x - b.x, a.y - b.y, a.z - b.z);
}

Vector operator-(Point p, Point q);

inline Vector operator*(const float s, const Vector a) {
	return Vector(a.x*s, a.y*s, a.z*s);
}

inline Vector operator/(const Vector a, const float s) {
	float recip = 1.0 / s;
	return Vector(a.x*recip, a.y*recip, a.z*recip);
}

inline float dot(const Vector a, const Vector b) {
	return (a.x*b.x + a.y*b.y + a.z*b.z);
}

inline Vector cross(const Vector a, const Vector b) {
	return Vector(a.y*b.z - a.z*b.y,
				  a.z*b.x - a.x*b.z,
				  a.x*b.y - a.y*b.x);
}

#endif